#define _LIBAC_OW_C_
#include <libac.h>


#define	__OW_PULSE(TB, TA)													\
		{																	\
			sbi(OW_PORT_DDR, OW_DQBIT);	/* drive low level (output port) */	\
			delay_us((TB));				/* for (TB) us					 */	\
			cbi(OW_PORT_DDR, OW_DQBIT);	/* release bus (input port)		 */	\
			delay_us((TA));				/* and wait for (TA) us			 */	\
		}


ow_device_st	ow_devices[OW_DEVICES];
uint8_t			ow_devices_active = 0;


/*
 * Generate a 1-Wire reset, return 0 if no presence detect pulse was found,
 * return 1 otherwise.
 * (NOTE: Does not handle alarm presence)
 */
uint8_t _ow_reset(void)
{
	cbi(OW_PORT, OW_DQBIT);			// be shure DQ is Tri-state (external
									// pull-up resistore drives high level)

	if (bit_is_clear(OW_PORT_IN, OW_DQBIT))
		return 0;					// detect low level on bus error, so
    								// we tell there is no device

	__OW_PULSE(480, 60);			// 1-Wire reset pulse

	if (bit_is_clear(OW_PORT_IN, OW_DQBIT)) {
		delay_us(420);				// detect low level on device present
		return 1;					// wait to fill out time slot
	} else {
		delay_us(420);				// sorry, there is no device
		return 0;					// wait to fill out time slot
	}
}

/*
 * Send a 1-Wire write bit. Provide 10us recovery time.
 */
void _ow_bit_write(uint8_t bit)
{
	cbi(OW_PORT, OW_DQBIT);			// be shure DQ is Tri-state (external
									// pull-up resistore drives high level)

	if (bit) {
		// Write `1` bit
		__OW_PULSE(6, 64);			// 1-Wire write `1` pulse
	} else {
		// Write `0` bit
		__OW_PULSE(60, 10);			// 1-Wire write `0` pulse
	}
}

/*
 * Read a bit from the 1-Wire bus and return it. Provide 10us recovery time.
 */
uint8_t _ow_bit_read(void)
{
	cbi(OW_PORT, OW_DQBIT);			// be shure DQ is Tri-state (external
									// pull-up resistore drives high level)

	__OW_PULSE(6, 9);				// 1-Wire read pulse

	if (bit_is_clear(OW_PORT_IN, OW_DQBIT)) {
		delay_us(55);				// detect low level on `0` bit
		return 0;					// wait to fill out time slot
	} else {
		delay_us(55);				// detect high level on `1` bit
		return 1;					// wait to fill out time slot
	}
}

/*
 * Send even one byte over the 1-Wire bus.
 */
uint8_t ow_byte_write(uint8_t byte)
{
	int i;
	for (i = 0; i < 8; i++) _ow_bit_write(byte & BV(i));
	return 1;
}

/*
 * Read even one byte from the 1-Wire bus and return it.
 */
uint8_t ow_byte_read(void)
{
	int i;
	uint8_t rb = 0;
	for (i = 0; i < 8; i++) rb |= _ow_bit_read() ? BV(i) : 0;
	return rb;
}


/*
 * generic bus (re-)initialization, returns number of devices are present
 */
uint8_t ow_init(void)
{
	cbi(OW_PORT,		OW_DQBIT);	// input port, Tri-state (default mode)
	cbi(OW_PORT_DDR,	OW_DQBIT);

	// erase device list
	ow_devices_active = 0;
	if (memset((void *)ow_devices, 0, sizeof(ow_devices))
			!= (void *)ow_devices)
		return 0;

	if (!_ow_reset())			// it seems, there is no device anymore
		return 0;

	return 1;
}


/*************************************EndOfFile***************************
$Id: $
vim:tw=78:ts=4:sw=4:ai:
*/
